BossTacticsUserData =
{
	CurrentPageSlug = "_about",
	Shown = false
}

BossTactics = { }

BossTactics.Layouts = { }

BossTactics.Pages =
{
  ["_notfound"] = BOSSTACTICS_LOC_NOTFOUNDPAGE,
  ["_about"] = BOSSTACTICS_LOC_ABOUTPAGE,
}

function BossTactics.OnLoad()
  -- Register an event to fire when the player logs in
  this:RegisterEvent('PLAYER_LOGIN');

  -- Create the popup menu
  BossTactics.DropDownMenu = CreateFrame("Frame", "BossTacticsDropDownMenu", BossTacticsFrame, "UIDropDownMenuTemplate");

  -- Set the localised title
  BossTacticsFrameTitle:SetText(BOSSTACTICS_LOC_FRAMETITLE)
end

function BossTactics.OnEvent(event)

    if event == "PLAYER_LOGIN" then

      BossTactics.SetCurrentPage(BossTacticsUserData.CurrentPageSlug)
      BossTactics.OnResize()
      
      BossTactics.RegisterSlashCommands()
      
      -- Show the frame if the current settings says so
      if BossTacticsUserData.Shown ~= false then
        BossTactics.ShowFrame()
      end
      
    end

end

function BossTactics.SetCurrentPage(pageSlug)

  local pageData = BossTactics.GetPage(pageSlug)
  
  BossTacticsUserData.CurrentPageSlug = pageSlug
  
  BossTactics.SetHTMLContents(pageData["body"])
  
end

function BossTactics.SetHTMLContents(text)
  BossTacticsHTMLFrame.text = text
  BossTacticsHTMLFrame:SetText(BossTacticsHTMLFrame.text)
end

function BossTactics.ToggleFrame()
	if BossTacticsFrame and BossTacticsFrame:IsVisible() then
		BossTactics.HideFrame()
	else
		BossTactics.ShowFrame()
	end
end

function BossTactics.ShowFrame()
	if BossTacticsFrame and not BossTacticsFrame:IsVisible() then
		BossTacticsFrame:Show()
	end
end

function BossTactics.HideFrame()
	if BossTacticsFrame and BossTacticsFrame:IsVisible() then
		BossTacticsFrame:Hide()
	end
end

function BossTactics.OnResize()
  BossTacticsHTMLFrame:SetWidth(BossTacticsFrame:GetWidth() - 50)
  BossTacticsHTMLFrame:SetText(BossTacticsHTMLFrame.text)
end

function BossTactics.OnShow()
  BossTacticsUserData.Shown = true
end

function BossTactics.OnHide()
		BossTacticsUserData.Shown = false
end

function BossTactics.PairsByKeys(t, f)
  local a = {}
  for n in pairs(t) do table.insert(a, n) end
  table.sort(a, f)
  local i = 0
  local iter = function()
    i = i + 1
    if a[i] == nil then return nil
    else return a[i], t[a[i]]
    end
  end
  return iter
end

function BossTactics.InitializeMenu(level)

	level = level or 1;
	
	local layout = nil
	
	if level == 1 then
	
    layout = BossTactics.Layouts
	
	else
	
		layout = UIDROPDOWNMENU_MENU_VALUE
		
	end

  if layout ~= nil then
    for key, child in BossTactics.PairsByKeys(layout) do
    
      local pageSlug = child["page"]
      local pageData = BossTactics.GetPage(pageSlug)
      
      if pageData ~= nil then
      
        local info = UIDropDownMenu_CreateInfo();
        info.hasArrow = child["layout"] ~= nil;
        info.notCheckable = true;
        info.text = pageData["name"];
        info.value = child["layout"]
        info.arg1 = pageSlug;
        info.func = BossTactics.MenuClick;
        
        if pageData == BossTactics.Pages["_notfound"] then
          info.notClickable = true;
        end
        
        UIDropDownMenu_AddButton(info, level);
        
      end
    end
  end

	if (level == 1) then
		local info = UIDropDownMenu_CreateInfo();
		
		info.hasArrow = false;
		info.notCheckable = true;
		info.text = BOSSTACTICS_LOC_ABOUT;
		info.func = BossTactics.MenuClickAbout;
		
		UIDropDownMenu_AddButton(info, level);
	end
	
end

function BossTactics.MenuClickAbout()
  BossTactics.SetCurrentPage("_about")
  BossTactics.CloseMenus()
end

function BossTactics.MenuButtonClick() 
  UIDropDownMenu_Initialize(BossTactics.DropDownMenu, BossTactics.InitializeMenu, "MENU");
	ToggleDropDownMenu(1, nil, BossTactics.DropDownMenu, BossTacticsMenuButton, 0, 0);
end

function BossTactics.MenuClick(pageSlug, pageData)
  BossTactics.SetCurrentPage(pageSlug)
  BossTactics.CloseMenus()
end

function BossTactics.CloseMenus()
	for i = 1, UIDROPDOWNMENU_MENU_LEVEL-1 do
		getglobal("DropDownList"..i):Hide()
	end
end

function BossTactics.OnHyperlinkShow(arg1, arg2, arg3)
	if strsub(arg1,1,5) == "page:" then
	  local pageSlug = strsub(arg1,6) or "_about"
      BossTactics.SetCurrentPage(pageSlug)
	end
end

function BossTactics.RegisterSlashCommands()
  SLASH_BOSSTACTICS1 = "/bosstactics";
  SLASH_BOSSTACTICS2 = "/bt";
  SlashCmdList["BOSSTACTICS"] = BossTactics.OnSlashCommand;
end

function BossTactics.OnSlashCommand(slashCommand)
  if slashCommand ~= '' then
    slashCommand = slashCommand:lower()
    if slashCommand == 'show' then
      BossTactics.ShowFrame()
    elseif slashCommand == 'hide' then
      BossTactics.HideFrame()
    elseif slashCommand == 'version' then
      DEFAULT_CHAT_FRAME:AddMessage(BOSSTACTICS_LOC_VERSION)
    else
      BossTactics.IllegalCommand()
    end
  else
    BossTactics.IllegalCommand()
  end
end

function BossTactics.IllegalCommand()
	if DEFAULT_CHAT_FRAME ~= nil then
		DEFAULT_CHAT_FRAME:AddMessage(BOSSTACTICS_LOC_COMMANDS)
		DEFAULT_CHAT_FRAME:AddMessage(BOSSTACTICS_LOC_HELPSHOW)
		DEFAULT_CHAT_FRAME:AddMessage(BOSSTACTICS_LOC_HELPHIDE)
		DEFAULT_CHAT_FRAME:AddMessage(BOSSTACTICS_LOC_HELPVERSION)
	end
end

function BossTactics:AddBook(slug, pages, layout, order)
  BossTactics.Pages[slug] = pages
  BossTactics.Layouts[order] = { ["page"] = slug..".default", ["layout"] = layout }
end

function BossTactics.GetPage(pageSlug)
  local s1, s2, s3 = strsplit(".", pageSlug)
  local page = nil
  
  if s2 == nil and s3 == nil then
    page = BossTactics.Pages[s1]
  elseif s3 == nil then
    if BossTactics.Pages[s1] ~= nil then
      page = BossTactics.Pages[s1][s2]
    end
  end

  if page == nil then
    return BossTactics.Pages["_notfound"]
  else
    return page
  end
  
end
